@@ -1,5 +1,6 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -18,8 +19,10 @@ Configuration options registration and useful routines.
 """
 
 import itertools
+import json
 import os
 
+from keystoneauth1 import loading
 from oslo_config import cfg
 from oslo_log import log
 from oslo_middleware import cors
@@ -89,7 +92,7 @@ api_opts = [
 js_impl_opt = cfg.StrOpt(
     'js_implementation',
     default='pyv8',
-    choices=['pyv8', 'v8eval'],
+    choices=['pyv8', 'v8eval', 'py_mini_racer'],
     help=_('The JavaScript implementation to be used by the std.javascript '
            'action to evaluate scripts.')
 )
@@ -110,6 +113,14 @@ rpc_response_timeout_opt = cfg.IntOpt(
     help=_('Seconds to wait for a response from a call.')
 )
 
+oslo_rpc_executor = cfg.StrOpt(
+    'oslo_rpc_executor',
+    default='eventlet',
+    choices=['eventlet', 'blocking', 'threading'],
+    help=_('Executor type used by Oslo Messaging framework. Defines how '
+           'Oslo Messaging based RPC subsystem processes incoming calls.')
+)
+
 expiration_token_duration = cfg.IntOpt(
     'expiration_token_duration',
     default=30,
@@ -172,6 +183,20 @@ engine_opts = [
                ' will be restored automatically. If this property is'
                ' set to a negative value Mistral will never be doing '
                ' this check.')
+    ),
+    cfg.IntOpt(
+        'execution_integrity_check_batch_size',
+        default=5,
+        min=1,
+        help=_('A number of task executions in RUNNING state that the'
+               ' execution integrity checker can process in a single'
+               ' iteration.')
+    ),
+    cfg.IntOpt(
+        'action_definition_cache_time',
+        default=60,
+        help=_('A number of seconds that indicates how long action '
+               'definitions should be stored in the local cache.')
     )
 ]
 
@@ -294,12 +319,43 @@ event_engine_opts = [
     ),
 ]
 
+notifier_opts = [
+    cfg.StrOpt(
+        'type',
+        choices=['local', 'remote'],
+        default='remote',
+        help=(
+            'Type of notifier. Use local to run the notifier within the '
+            'engine server. Use remote if the notifier is launched as '
+            'a separate server to process events.'
+        )
+    ),
+    cfg.StrOpt(
+        'host',
+        default='0.0.0.0',
+        help=_('Name of the notifier node. This can be an opaque '
+               'identifier. It is not necessarily a hostname, '
+               'FQDN, or IP address.')
+    ),
+    cfg.StrOpt(
+        'topic',
+        default='mistral_notifier',
+        help=_('The message topic that the notifier server listens on.')
+    ),
+    cfg.ListOpt(
+        'notify',
+        item_type=json.loads,
+        bounds=True,
+        help=_('List of publishers to publish notification.')
+    )
+]
+
 execution_expiration_policy_opts = [
     cfg.IntOpt(
         'evaluation_interval',
         help=_('How often will the executions be evaluated '
                '(in minutes). For example for value 120 the interval '
-               'will be 2 hours (every 2 hours).'
+               'will be 2 hours (every 2 hours). '
                'Note that only final state executions will be removed: '
                '( SUCCESS / ERROR / CANCELLED ).')
     ),
@@ -313,12 +369,12 @@ execution_expiration_policy_opts = [
     cfg.IntOpt(
         'max_finished_executions',
         default=0,
-        help=_('The maximum number of finished workflow executions'
-               'to be stored. For example when max_finished_executions = 100,'
-               'only the 100 latest finished executions will be preserved.'
-               'This means that even unexpired executions are eligible'
-               'for deletion, to decrease the number of executions in the'
-               'database. The default value is 0. If it is set to 0,'
+        help=_('The maximum number of finished workflow executions '
+               'to be stored. For example when max_finished_executions = 100, '
+               'only the 100 latest finished executions will be preserved. '
+               'This means that even unexpired executions are eligible '
+               'for deletion, to decrease the number of executions in the '
+               'database. The default value is 0. If it is set to 0, '
                'this constraint won\'t be applied.')
     ),
     cfg.IntOpt(
@@ -326,11 +382,52 @@ execution_expiration_policy_opts = [
         default=0,
         help=_('Size of batch of expired executions to be deleted.'
                'The default value is 0. If it is set to 0, '
-               'size of batch is total number of expired executions'
+               'size of batch is total number of expired executions '
                'that is going to be deleted.')
     )
 ]
 
+action_heartbeat_opts = [
+    cfg.IntOpt(
+        'max_missed_heartbeats',
+        min=0,
+        default=15,
+        help=_('The maximum amount of missed heartbeats to be allowed. '
+               'If set to 0 then this feature is disabled. '
+               'See check_interval for more details.')
+    ),
+    cfg.IntOpt(
+        'check_interval',
+        min=0,
+        default=20,
+        help=_('How often (in seconds) action executions are checked. '
+               'For example when check_interval is 10, check action '
+               'executions every 10 seconds. When the checker runs it will '
+               'transit all running action executions to error if the last '
+               'heartbeat received is older than 10 * max_missed_heartbeats '
+               'seconds. If set to 0 then this feature is disabled.')
+    ),
+    cfg.IntOpt(
+        'batch_size',
+        min=0,
+        default=10,
+        help=_('The maximum number of action executions processed during '
+               'one iteration of action execution heartbeat checker. If set '
+               'to 0 then there is no limit.')
+    ),
+    cfg.IntOpt(
+        'first_heartbeat_timeout',
+        min=0,
+        default=3600,
+        help=_('The first heartbeat is handled differently, to provide a '
+               'grace period in case there is no available executor to handle '
+               'the action execution. For example when '
+               'first_heartbeat_timeout = 3600, wait 3600 seconds before '
+               'closing the action executions that never received a heartbeat.'
+               )
+    )
+]
+
 coordination_opts = [
     cfg.StrOpt(
         'backend_url',
@@ -374,7 +471,12 @@ keycloak_oidc_opts = [
         'insecure',
         default=False,
         help=_('If True, SSL/TLS certificate verification is disabled')
-    )
+    ),
+    cfg.StrOpt(
+        'user_info_endpoint_url',
+        default='/realms/%s/protocol/openid-connect/userinfo',
+        help='Endpoint against which authorization will be performed'
+    ),
 ]
 
 openstack_actions_opts = [
@@ -411,6 +513,76 @@ os_actions_mapping_path = cfg.StrOpt(
          'directory or absolute.'
 )
 
+yaql_opts = [
+    cfg.IntOpt(
+        'limit_iterators',
+        default=-1,
+        min=-1,
+        help=_('Limit iterators by the given number of elements. When set, '
+               'each time any function declares its parameter to be iterator, '
+               'that iterator is modified to not produce more than a given '
+               'number of items. If not set (or set to -1) the result data is '
+               'allowed to contain endless iterators that would cause errors '
+               'if the result where to be serialized.')
+    ),
+    cfg.IntOpt(
+        'memory_quota',
+        default=-1,
+        min=-1,
+        help=_('The memory usage quota (in bytes) for all data produced by '
+               'the expression (or any part of it). -1 means no limitation.')
+    ),
+    cfg.BoolOpt(
+        'convert_input_data',
+        default=True,
+        help=_('Enables input data conversion for YAQL expressions. If set '
+               'to True then YAQL will convert mutable data structures '
+               '(lists, dicts, sets) into their immutable versions. That '
+               'will allow them to work with some constructs that require '
+               'hashable types even if elements are not hashable. For '
+               'example, it will be possible to put dicts into a set. '
+               'Although it conflicts with the base principles of such '
+               'collections (e.g. we cannot put a non-hashable type into '
+               'a set just because otherwise it will not work correctly) the '
+               'YAQL library itself allows this. '
+               'Disabling input data conversion may give significant '
+               'performance boost if the input data for an expression is '
+               'large.')
+    ),
+    cfg.BoolOpt(
+        'convert_tuples_to_lists',
+        default=True,
+        help=_('When set to true, yaql converts all tuples in the expression '
+               'result to lists.')
+    ),
+    cfg.BoolOpt(
+        'convert_sets_to_lists',
+        default=False,
+        help=_('When set to true, yaql converts all sets in the expression '
+               'result to lists. Otherwise the produced result may contain '
+               'sets that are not JSON-serializable.')
+    ),
+    cfg.BoolOpt(
+        'iterable_dicts',
+        default=False,
+        help=_('When set to true, dictionaries are considered to be iterable '
+               'and iteration over dictionaries produces their keys (as in '
+               'Python and yaql 0.2).')
+    ),
+    cfg.StrOpt(
+        'keyword_operator',
+        default='=>',
+        help=_('Allows one to configure keyword/mapping symbol. '
+               'Ability to pass named arguments can be disabled altogether '
+               'if empty string is provided.')
+    ),
+    cfg.BoolOpt(
+        'allow_delegates',
+        default=False,
+        help=_('Enables or disables delegate expression parsing.')
+    )
+]
+
 CONF = cfg.CONF
 
 API_GROUP = 'api'
@@ -419,12 +591,16 @@ EXECUTOR_GROUP = 'executor'
 SCHEDULER_GROUP = 'scheduler'
 CRON_TRIGGER_GROUP = 'cron_trigger'
 EVENT_ENGINE_GROUP = 'event_engine'
+NOTIFIER_GROUP = 'notifier'
 PECAN_GROUP = 'pecan'
 COORDINATION_GROUP = 'coordination'
 EXECUTION_EXPIRATION_POLICY_GROUP = 'execution_expiration_policy'
+ACTION_HEARTBEAT_GROUP = 'action_heartbeat'
 PROFILER_GROUP = profiler.list_opts()[0][0]
 KEYCLOAK_OIDC_GROUP = "keycloak_oidc"
 OPENSTACK_ACTIONS_GROUP = 'openstack_actions'
+YAQL_GROUP = "yaql"
+KEYSTONE_GROUP = "keystone"
 
 
 CONF.register_opt(wf_trace_log_name_opt)
@@ -432,6 +608,7 @@ CONF.register_opt(auth_type_opt)
 CONF.register_opt(js_impl_opt)
 CONF.register_opt(rpc_impl_opt)
 CONF.register_opt(rpc_response_timeout_opt)
+CONF.register_opt(oslo_rpc_executor)
 CONF.register_opt(expiration_token_duration)
 
 CONF.register_opts(api_opts, group=API_GROUP)
@@ -443,12 +620,19 @@ CONF.register_opts(
     execution_expiration_policy_opts,
     group=EXECUTION_EXPIRATION_POLICY_GROUP
 )
+CONF.register_opts(
+    action_heartbeat_opts,
+    group=ACTION_HEARTBEAT_GROUP
+)
 CONF.register_opts(event_engine_opts, group=EVENT_ENGINE_GROUP)
+CONF.register_opts(notifier_opts, group=NOTIFIER_GROUP)
 CONF.register_opts(pecan_opts, group=PECAN_GROUP)
 CONF.register_opts(coordination_opts, group=COORDINATION_GROUP)
 CONF.register_opts(profiler_opts, group=PROFILER_GROUP)
 CONF.register_opts(keycloak_oidc_opts, group=KEYCLOAK_OIDC_GROUP)
 CONF.register_opts(openstack_actions_opts, group=OPENSTACK_ACTIONS_GROUP)
+CONF.register_opts(yaql_opts, group=YAQL_GROUP)
+loading.register_session_conf_options(CONF, KEYSTONE_GROUP)
 
 CLI_OPTS = [
     use_debugger_opt,
@@ -463,6 +647,7 @@ default_group_opts = itertools.chain(
         js_impl_opt,
         rpc_impl_opt,
         rpc_response_timeout_opt,
+        oslo_rpc_executor,
         expiration_token_duration
     ]
 )
@@ -488,12 +673,15 @@ def list_opts():
         (EVENT_ENGINE_GROUP, event_engine_opts),
         (SCHEDULER_GROUP, scheduler_opts),
         (CRON_TRIGGER_GROUP, cron_trigger_opts),
+        (NOTIFIER_GROUP, notifier_opts),
         (PECAN_GROUP, pecan_opts),
         (COORDINATION_GROUP, coordination_opts),
         (EXECUTION_EXPIRATION_POLICY_GROUP, execution_expiration_policy_opts),
         (PROFILER_GROUP, profiler_opts),
         (KEYCLOAK_OIDC_GROUP, keycloak_oidc_opts),
         (OPENSTACK_ACTIONS_GROUP, openstack_actions_opts),
+        (YAQL_GROUP, yaql_opts),
+        (ACTION_HEARTBEAT_GROUP, action_heartbeat_opts),
         (None, default_group_opts)
     ]
 
