@@ -15,8 +15,8 @@
 
 from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
-from mistral.engine import action_queue
 from mistral.engine import base
+from mistral.engine import post_tx_queue
 from mistral.engine import workflow_handler as wf_handler
 from mistral import expressions
 from mistral.services import scheduler
@@ -28,8 +28,9 @@ import six
 
 _CONTINUE_TASK_PATH = 'mistral.engine.policies._continue_task'
 _COMPLETE_TASK_PATH = 'mistral.engine.policies._complete_task'
-_FAIL_IF_INCOMPLETE_TASK_PATH = \
+_FAIL_IF_INCOMPLETE_TASK_PATH = (
     'mistral.engine.policies._fail_task_if_incomplete'
+)
 
 
 def _log_task_delay(task_ex, delay_sec):
@@ -259,9 +260,9 @@ class WaitAfterPolicy(base.TaskPolicy):
         end_state = task_ex.state
         end_state_info = task_ex.state_info
 
-        # TODO(rakhmerov): Policies probably needs to have tasks.Task
-        # interface in order to change manage task state safely.
-        # Set task state to 'DELAYED'.
+        # TODO(rakhmerov): Policies probably need to have tasks.Task
+        # interface in order to manage task state safely.
+        # Set task state to 'RUNNING_DELAYED'.
         task_ex.state = states.RUNNING_DELAYED
         task_ex.state_info = (
             'Suspended by wait-after policy for %s seconds' % self.delay
@@ -391,6 +392,7 @@ class RetryPolicy(base.TaskPolicy):
         _log_task_delay(task_ex, self.delay)
 
         data_flow.invalidate_task_execution_result(task_ex)
+
         task_ex.state = states.RUNNING_DELAYED
 
         policy_context['retry_no'] = retry_no + 1
@@ -403,6 +405,15 @@ class RetryPolicy(base.TaskPolicy):
             task_ex_id=task_ex.id,
         )
 
+    @staticmethod
+    def refresh_runtime_context(task_ex):
+        runtime_context = task_ex.runtime_context or {}
+        retry_task_policy = runtime_context.get('retry_task_policy')
+
+        if retry_task_policy:
+            retry_task_policy['retry_no'] = 0
+            task_ex.runtime_context['retry_task_policy'] = retry_task_policy
+
 
 class TimeoutPolicy(base.TaskPolicy):
     _schema = {
@@ -500,7 +511,7 @@ class ConcurrencyPolicy(base.TaskPolicy):
 
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 def _continue_task(task_ex_id):
     from mistral.engine import task_handler
 
@@ -509,20 +520,18 @@ def _continue_task(task_ex_id):
 
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 def _complete_task(task_ex_id, state, state_info):
     from mistral.engine import task_handler
 
     with db_api.transaction():
-        task_handler.complete_task(
-            db_api.get_task_execution(task_ex_id),
-            state,
-            state_info
-        )
+        task_ex = db_api.get_task_execution(task_ex_id)
+
+        task_handler.complete_task(task_ex, state, state_info)
 
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 def _fail_task_if_incomplete(task_ex_id, timeout):
     from mistral.engine import task_handler
 
@@ -532,8 +541,4 @@ def _fail_task_if_incomplete(task_ex_id, timeout):
         if not states.is_completed(task_ex.state):
             msg = 'Task timed out [timeout(s)=%s].' % timeout
 
-            task_handler.complete_task(
-                db_api.get_task_execution(task_ex_id),
-                states.ERROR,
-                msg
-            )
+            task_handler.complete_task(task_ex, states.ERROR, msg)
