@@ -1,5 +1,6 @@
 # Copyright 2016 - Nokia Networks.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -15,6 +16,8 @@
 
 import abc
 import copy
+import json
+from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
 import six
@@ -23,8 +26,12 @@ from mistral.db.v2 import api as db_api
 from mistral.engine import actions
 from mistral.engine import dispatcher
 from mistral.engine import policies
+from mistral.engine import post_tx_queue
+from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions as exc
 from mistral import expressions as expr
+from mistral.notifiers import base as notif
+from mistral.notifiers import notification_events as events
 from mistral import utils
 from mistral.utils import wf_trace
 from mistral.workflow import base as wf_base
@@ -57,6 +64,23 @@ class Task(object):
         self.created = False
         self.state_changed = False
 
+    def notify(self, old_task_state, new_task_state):
+        publishers = self.wf_ex.params.get('notify')
+
+        if not publishers and not isinstance(publishers, list):
+            return
+
+        notifier = notif.get_notifier(cfg.CONF.notifier.type)
+        event = events.identify_task_event(old_task_state, new_task_state)
+
+        notifier.notify(
+            self.task_ex.id,
+            self.task_ex.to_dict(),
+            event,
+            self.task_ex.updated_at,
+            publishers
+        )
+
     def is_completed(self):
         return self.task_ex and states.is_completed(self.task_ex.state)
 
@@ -96,6 +120,23 @@ class Task(object):
 
         This method puts task to a waiting state.
         """
+
+        # NOTE(rakhmerov): using named locks may cause problems under load
+        # with MySQL that raises a lot of deadlocks in case of high
+        # parallelism so it makes sense to do a fast check if the object
+        # already exists in DB outside of the lock.
+        if not self.task_ex:
+            t_execs = db_api.get_task_executions(
+                workflow_execution_id=self.wf_ex.id,
+                unique_key=self.unique_key,
+                state=states.WAITING
+            )
+
+            self.task_ex = t_execs[0] if t_execs else None
+
+        if self.task_ex:
+            return
+
         with db_api.named_lock(self.unique_key):
             if not self.task_ex:
                 t_execs = db_api.get_task_executions(
@@ -145,7 +186,8 @@ class Task(object):
                 return False
 
             self.task_ex = task_ex
-            self.task_ex.state_info = state_info
+            self.task_ex.state_info = json.dumps(state_info) \
+                if isinstance(state_info, dict) else state_info
             self.state_changed = True
 
             if processed is not None:
@@ -158,7 +200,7 @@ class Task(object):
                  self.task_ex.id,
                  cur_state,
                  state,
-                 state_info)
+                 self.task_ex.state_info)
             )
 
         return True
@@ -177,8 +219,15 @@ class Task(object):
 
         assert self.task_ex
 
+        # Record the current task state.
+        old_task_state = self.task_ex.state
+
         # Ignore if task already completed.
         if self.is_completed():
+            # Publish task event again so subscribers know
+            # task completed state is being processed again.
+            self.notify(old_task_state, self.task_ex.state)
+
             return
 
         # If we were unable to change the task state it means that it was
@@ -205,6 +254,9 @@ class Task(object):
         # If workflow is paused we shouldn't schedule new commands
         # and mark task as processed.
         if states.is_paused(self.wf_ex.state):
+            # Publish task event even if the workflow is paused.
+            self.notify(old_task_state, self.task_ex.state)
+
             return
 
         wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
@@ -216,8 +268,25 @@ class Task(object):
         # upon its completion.
         self.task_ex.processed = True
 
+        self.register_workflow_completion_check()
+
+        # Publish task event.
+        self.notify(old_task_state, self.task_ex.state)
+
         dispatcher.dispatch_workflow_commands(self.wf_ex, cmds)
 
+    def register_workflow_completion_check(self):
+        wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
+
+        # Register an asynchronous command to check workflow completion
+        # in a separate transaction if the task may potentially lead to
+        # workflow completion.
+        def _check():
+            wf_handler.check_and_complete(self.wf_ex.id)
+
+        if wf_ctrl.may_complete_workflow(self.task_ex):
+            post_tx_queue.register_operation(_check, in_tx=True)
+
     @profiler.trace('task-update')
     def update(self, state, state_info=None):
         """Update task and set specified state.
@@ -230,8 +299,15 @@ class Task(object):
 
         assert self.task_ex
 
+        # Record the current task state.
+        old_task_state = self.task_ex.state
+
         # Ignore if task already completed.
         if states.is_completed(self.task_ex.state):
+            # Publish task event again so subscribers know
+            # task completed state is being processed again.
+            self.notify(old_task_state, self.task_ex.state)
+
             return
 
         # Update only if state transition is valid.
@@ -247,6 +323,12 @@ class Task(object):
 
         self.set_state(state, state_info)
 
+        if states.is_completed(self.task_ex.state):
+            self.register_workflow_completion_check()
+
+        # Publish event.
+        self.notify(old_task_state, self.task_ex.state)
+
     def _before_task_start(self):
         policies_spec = self.task_spec.get_policies()
 
@@ -290,6 +372,22 @@ class Task(object):
 
         self.created = True
 
+    def _get_safe_rerun(self):
+        safe_rerun = self.task_spec.get_safe_rerun()
+
+        if safe_rerun is not None:
+            return safe_rerun
+
+        task_defaults = self.wf_spec.get_task_defaults()
+
+        if task_defaults:
+            default_safe_rerun = task_defaults.get_safe_rerun()
+
+            if default_safe_rerun is not None:
+                return default_safe_rerun
+
+        return False
+
     def _get_action_defaults(self):
         action_name = self.task_spec.get_action_name()
 
@@ -310,11 +408,16 @@ class RegularTask(Task):
     @profiler.trace('regular-task-on-action-complete', hide_args=True)
     def on_action_complete(self, action_ex):
         state = action_ex.state
-        # TODO(rakhmerov): Here we can define more informative messages
+        # TODO(rakhmerov): Here we can define more informative messages for
         # cases when action is successful and when it's not. For example,
         # in state_info we can specify the cause action.
-        state_info = (None if state == states.SUCCESS
-                      else action_ex.output.get('result'))
+
+        if state == states.SUCCESS:
+            state_info = None
+        else:
+            action_result = action_ex.output.get('result')
+
+            state_info = str(action_result) if action_result else None
 
         self.complete(state, state_info)
 
@@ -338,6 +441,9 @@ class RegularTask(Task):
 
         self._create_task_execution()
 
+        # Publish event.
+        self.notify(None, self.task_ex.state)
+
         LOG.debug(
             'Starting task [workflow=%s, task=%s, init_state=%s]',
             self.wf_ex.name,
@@ -365,8 +471,14 @@ class RegularTask(Task):
                 'Rerunning succeeded tasks is not supported.'
             )
 
+        # Record the current task state.
+        old_task_state = self.task_ex.state
+
         self.set_state(states.RUNNING, None, processed=False)
 
+        # Publish event.
+        self.notify(old_task_state, self.task_ex.state)
+
         self._update_inbound_context()
         self._update_triggered_by()
         self._reset_actions()
@@ -419,12 +531,15 @@ class RegularTask(Task):
         action.schedule(
             input_dict,
             target,
-            safe_rerun=self.task_spec.get_safe_rerun(),
+            safe_rerun=self._get_safe_rerun(),
             timeout=self._get_timeout()
         )
 
     @profiler.trace('regular-task-get-target', hide_args=True)
     def _get_target(self, input_dict):
+        if not self.task_spec.get_target():
+            return None
+
         ctx_view = data_flow.ContextView(
             input_dict,
             self.ctx,
@@ -440,7 +555,11 @@ class RegularTask(Task):
 
     @profiler.trace('regular-task-get-action-input', hide_args=True)
     def _get_action_input(self, ctx=None):
-        input_dict = self._evaluate_expression(self.task_spec.get_input(), ctx)
+        input_spec = self.task_spec.get_input()
+
+        input_dict = (
+            self._evaluate_expression(input_spec, ctx) if input_spec else {}
+        )
 
         if not isinstance(input_dict, dict):
             raise exc.InputException(
@@ -464,10 +583,7 @@ class RegularTask(Task):
             self.wf_ex.input
         )
 
-        return expr.evaluate_recursively(
-            expression,
-            ctx_view
-        )
+        return expr.evaluate_recursively(expression, ctx_view)
 
     def _build_action(self):
         action_name = self.task_spec.get_action_name()
@@ -602,7 +718,7 @@ class WithItemsTask(RegularTask):
                 input_dict,
                 target,
                 index=i,
-                safe_rerun=self.task_spec.get_safe_rerun(),
+                safe_rerun=self._get_safe_rerun(),
                 timeout=self._get_timeout()
             )
 
