@@ -63,7 +63,6 @@ class KombuRPCServer(rpc_base.RPCServer, kombu_base.Base):
         self.exchange = CONF.control_exchange
         # TODO(rakhmerov): We shouldn't rely on any properties related
         # to oslo.messaging. Only "transport_url" should matter.
-        self.virtual_host = CONF.oslo_messaging_rabbit.rabbit_virtual_host
         self.durable_queue = CONF.oslo_messaging_rabbit.amqp_durable_queues
         self.auto_delete = CONF.oslo_messaging_rabbit.amqp_auto_delete
         self.routing_key = self.topic
@@ -84,7 +83,7 @@ class KombuRPCServer(rpc_base.RPCServer, kombu_base.Base):
         """Return whether server is running."""
         return self._running.is_set()
 
-    def run(self, executor='blocking'):
+    def run(self, executor='eventlet'):
         if self._thread is None:
             self._thread = threading.Thread(target=self._run, args=(executor,))
             self._thread.daemon = True
@@ -104,7 +103,7 @@ class KombuRPCServer(rpc_base.RPCServer, kombu_base.Base):
                     host.port,
                     host.username,
                     host.password,
-                    self.virtual_host,
+                    self._hosts.virtual_host,
                 )
 
                 conn = kombu.connections[self.conn].acquire(block=True)
@@ -147,8 +146,8 @@ class KombuRPCServer(rpc_base.RPCServer, kombu_base.Base):
                             self.stop()
 
                             LOG.info(
-                                "Server with id='%w' stopped.",
-                                self.server_id
+                                "Server with id='{}' stopped."
+                                .format(self.server_id)
                             )
 
                             return
@@ -286,7 +285,7 @@ class KombuRPCServer(rpc_base.RPCServer, kombu_base.Base):
 
         executor_opts = {}
 
-        if executor == 'threading':
+        if executor != 'blocking':
             executor_opts['max_workers'] = self._executor_threads
 
         self._worker = mgr.driver(**executor_opts)
