@@ -91,31 +91,6 @@ class JinjaEvaluatorTest(base.BaseTest):
         res = self._evaluator.evaluate("_.server.status == 'ACTIVE'", DATA)
         self.assertTrue(res)
 
-    def test_wrong_expression(self):
-        res = self._evaluator.evaluate("_.status == 'Invalid value'", DATA)
-        self.assertFalse(res)
-
-        # One thing to note about Jinja is that by default it would not raise
-        # an exception on KeyError inside the expression, it will consider
-        # value to be None. Same with NameError, it won't return an original
-        # expression (which by itself seems confusing). Jinja allows us to
-        # change behavior in both cases by switching to StrictUndefined, but
-        # either one or the other will surely suffer.
-
-        self.assertRaises(
-            exc.JinjaEvaluationException,
-            self._evaluator.evaluate,
-            '_.wrong_key',
-            DATA
-        )
-
-        self.assertRaises(
-            exc.JinjaEvaluationException,
-            self._evaluator.evaluate,
-            'invalid_expression_string',
-            DATA
-        )
-
     def test_select_result(self):
         res = self._evaluator.evaluate(
             '_.servers|selectattr("name", "equalto", "ubuntu")',
@@ -241,7 +216,7 @@ class JinjaEvaluatorTest(base.BaseTest):
             'type': task.type,
             'workflow_execution_id': task.workflow_execution_id,
             'created_at': task.created_at.isoformat(' '),
-            'updated_at': task.updated_at.isoformat(' ')
+            'updated_at': task.updated_at.isoformat(' '),
         }, result)
 
     @mock.patch('mistral.db.v2.api.get_task_executions')
@@ -345,7 +320,8 @@ class JinjaEvaluatorTest(base.BaseTest):
             'input': wf_ex.input,
             'params': wf_ex.params,
             'created_at': wf_ex.created_at.isoformat(' '),
-            'updated_at': wf_ex.updated_at.isoformat(' ')
+            'updated_at': wf_ex.updated_at.isoformat(' '),
+            'root_execution_id': wf_ex.root_execution_id
         }, result)
 
     def test_executions(self):
@@ -483,7 +459,8 @@ class JinjaEvaluatorTest(base.BaseTest):
             'input': wf_ex.input,
             'params': wf_ex.params,
             'created_at': wf_ex.created_at.isoformat(' '),
-            'updated_at': wf_ex.updated_at.isoformat(' ')
+            'updated_at': wf_ex.updated_at.isoformat(' '),
+            'root_execution_id': wf_ex.root_execution_id
         }, result)
 
 
@@ -597,3 +574,36 @@ class InlineJinjaEvaluatorTest(base.BaseTest):
         self.assertRaises(exc.JinjaEvaluationException,
                           self._evaluator.validate,
                           {'a': 1})
+
+    def test_wrong_expression(self):
+        res = self._evaluator.evaluate("{{ _.status == 'Invalid value' }}",
+                                       DATA)
+        self.assertFalse(res)
+
+        # One thing to note about Jinja is that by default it would not raise
+        # an exception on KeyError inside the expression, it will consider
+        # value to be None. Same with NameError, it won't return an original
+        # expression (which by itself seems confusing). Jinja allows us to
+        # change behavior in both cases by switching to StrictUndefined, but
+        # either one or the other will surely suffer.
+
+        self.assertRaises(
+            exc.JinjaEvaluationException,
+            self._evaluator.evaluate,
+            '{{ _.wrong_key }}',
+            DATA
+        )
+
+        self.assertRaises(
+            exc.JinjaEvaluationException,
+            self._evaluator.evaluate,
+            '{{ invalid_expression_string }}',
+            DATA
+        )
+
+        self.assertRaises(
+            exc.JinjaEvaluationException,
+            self._evaluator.evaluate,
+            '!! {{ _.nonexistent_variable }} !!',
+            DATA
+        )
