@@ -28,7 +28,6 @@ from mistral import context
 from mistral import exceptions
 
 CONF = cfg.CONF
-CONF.register_opt(cfg.IntOpt('timeout'), group='keystone_authtoken')
 
 
 def client():
@@ -118,8 +117,10 @@ def _admin_client(trust_id=None):
         kwargs = {}
 
         if trust_id:
-            # Remove project_name and project_id, since we need a trust scoped
-            # auth object
+            # Remove domain_id, domain_name, project_name and project_id,
+            # since we need a trust scoped auth object
+            kwargs['domain_id'] = None
+            kwargs['domain_name'] = None
             kwargs['project_name'] = None
             kwargs['project_domain_name'] = None
             kwargs['project_id'] = None
@@ -132,7 +133,7 @@ def _admin_client(trust_id=None):
         )
         sess = loading.load_session_from_conf_options(
             CONF,
-            'keystone_authtoken',
+            'keystone',
             auth=auth
         )
 
@@ -222,8 +223,12 @@ def obtain_service_catalog(ctx):
                 "'trust_id' must be provided in the admin context."
             )
 
-        trust_client = client_for_trusts(ctx.trust_id)
-        token_data = trust_client.tokens.get_token_data(
+        # trust_client = client_for_trusts(ctx.trust_id)
+        # Using trust client, it can't validate token
+        # when cron trigger running because keystone policy
+        # don't allow do this. So we need use admin client to
+        # get token data
+        token_data = _admin_client().tokens.get_token_data(
             token,
             include_catalog=True
         )
@@ -247,11 +252,11 @@ def obtain_service_catalog(ctx):
     return service_catalog
 
 
-def get_keystone_endpoint_v2():
+def get_keystone_endpoint():
     return get_endpoint_for_project('keystone', service_type='identity')
 
 
-def get_keystone_url_v2():
+def get_keystone_url():
     return get_endpoint_for_project('keystone', service_type='identity').url
 
 
@@ -288,7 +293,7 @@ def get_admin_session():
 
         return loading.load_session_from_conf_options(
             CONF,
-            'keystone_authtoken',
+            'keystone',
             auth=auth
         )
 
