@@ -27,6 +27,7 @@ from wsme import exc as wsme_exc
 
 
 from mistral import context as auth_ctx
+from mistral.db import utils as db_utils
 from mistral.db.v2.sqlalchemy import api as db_api
 from mistral import exceptions as exc
 
@@ -232,7 +233,7 @@ def get_all(list_cls, cls, get_all_function, get_function,
     return list_cls.convert_with_links(
         rest_resources,
         limit,
-        pecan.request.host_url,
+        pecan.request.application_url,
         sort_keys=','.join(sort_keys),
         sort_dirs=','.join(sort_dirs),
         fields=','.join(fields) if fields else '',
@@ -251,8 +252,16 @@ class MistralRetrying(tenacity.Retrying):
 def create_db_retry_object():
     return MistralRetrying(
         retry=tenacity.retry_if_exception_type(
-            (sa.exc.OperationalError, db_exc.DBConnectionError)
+            (
+                sa.exc.OperationalError,
+                db_exc.DBDeadlock,
+                db_exc.DBConnectionError
+            )
         ),
         stop=tenacity.stop_after_attempt(10),
-        wait=tenacity.wait_incrementing(increment=0.2)  # 0.2 seconds
+        wait=tenacity.wait_incrementing(increment=0.5)  # 0.5 seconds
     )
+
+
+def rest_retry_on_db_error(func):
+    return db_utils.retry_on_db_error(func, create_db_retry_object())
